!> \file climat-hudson_mod.f90
!! Module pour le le calcul du climat des experiences Heino avec shelf
!<

!> \namespace  climat_hudson
!! Module pour le calcule du climat des experiences Heino avec shelf
!! \author ...
!! \date ...
!! @note Used module
!! @note   - use module3D_phy
!<
module  climat_hudson               ! pour le climat des experiences Heino avec shelf


use module3d_phy
implicit none

real, dimension(nx,ny) :: dist_heino     !< distance au centre
real,dimension(nx,ny) ::  xcc , ycc      !< coordeonnes en m
real :: bilmin                           !< bilan min
real :: bilmax                           !< bilan max 
real :: coefbil
real :: Rcal                             !< rayon de la calotte
real :: Tsmin                            !< Temperature min
real :: St                               !< gradient horizontal de temperature
real :: time_change                      !< temps apres lequel on met un changement
real :: deltaTann                        !< changement de temeprature
real :: ratiobil                         !< changement de bilan (facteur)


contains
!______________________________________________________________________________

!>SUBROUTINE: input_clim
!! Initialise la distance au centre
!<

subroutine input_clim                  ! initialise la distance au centre

real :: xcentre !< dist centre en km
real :: ycentre
real :: coefbil

!cdc if (itracebug.eq.1)  call tracebug(' Heino: entree dans routine input_clim')

xcentre=((NX-1)/2)*DX/1000.
ycentre=((NY-1)/2)*DX/1000.

!jalv: rayon de la calotte virtuelle dont ses bords sont determins par le bilan de masse, car le domaine sur Hudson est maintenant ( differnece de Heino) entierement CONTINENTALE(sauf la baie)
 
Rcal=2400.    ! en km mais la distance l'est aussi

print*,'xcentre,ycentre', xcentre,ycentre

do j=1,ny
   do i=1,nx
     xcc(i,j)=(i-1)*dx
     ycc(i,j)=(j-1)*dy
   end do
end do

xcc(:,:)=xcc(:,:)/1000.
ycc(:,:)=ycc(:,:)/1000.


dist_heino(:,:)=(xcc(:,:)-xcentre)**2+(ycc(:,:)-ycentre)**2
dist_heino(:,:)=dist_heino(:,:)**0.5

write(42,*)
write(42,*)'bilan Heino------------------------------------'
write(42,*)'initialisation distance au centre en km'

bilmin=0.15   ! en m/an
bilmax=0.30    ! en m/an

coefbil=(bilmax-bilmin)/Rcal

!jalv bm(:,:)=bilmin+coefbil*dist_heino(:,:)

acc(:,:)=bilmin+coefbil*dist_heino(:,:)

!Acc(:,:)=bm(:,:)

!jalv:
! Hudson: bords de la calotte definies par bilan de masse=0

where (dist_heino(:,:).gt.2400.) acc(:,:)=0.

abl(:,:)=0.

!jalv:
! Hudson: bords de la calotte definies par une ablation tres tres forte Abl=10.

where (dist_heino(:,:).gt.2400.) abl(:,:)=-20.

!where (flot(:,:)) abl(:,:)=0.
!where (flot(:,:)) acc(:,:)=0.

!where (mk(:,:).lt.9) acc(:,:)=0.   ! Heino: bilan de masse=0 dans l'ocean
!where (mk(:,:).lt.9) abl(:,:)=0.

!jalv: test pour que le bilan de masse soit 0 sur l'ocean plus profond de 600 m:
!      c'est le bon critere??:
where (sealevel-BSOC(:,:).gt.600) acc(:,:)=0.
where (sealevel-BSOC(:,:).gt.600) abl(:,:)=0.

where ((sealevel-BSOC(:,:).gt.100).and.(dist_heino(:,:).gt.2500.)) abl(:,:)=-1.
where ((sealevel-BSOC(:,:).gt.100).and.(dist_heino(:,:).gt.2600.)) abl(:,:)=-5.
where ((sealevel-BSOC(:,:).gt.100).and.(dist_heino(:,:).gt.2650.)) abl(:,:)=-10.
where ((sealevel-BSOC(:,:).gt.100).and.(dist_heino(:,:).gt.2700.)) abl(:,:)=-20.

bm(:,:)=acc(:,:)+abl(:,:)

write(42,*) 'bilmin,bilmax (m/an)',bilmin,bilmax

Tsmin=-40.0    ! run standard

!jalv: pour que la temp soit pas positive dans les bords de la calotte dans l'experience 
!      Hudson il faut une Tmin plus basse: on prens celle du run T1-Heino
!      (pas forcement vrai)

!Tsmin=-50.0    ! run T1
St=2.5e-9

write(42,*)'Temperature, Tsmin (deg C), gradient (deg/km-3)',Tsmin,St

Tann(:,:)=Tsmin+St*dist_heino(:,:)**3


! Attention dans la version shelf on utilise mk et pas mk0
!where (mk(:,:).eq.1) Tann(:,:)=0.
!where (flot(:,:)) Tann(:,:)=0.

!jalv: test pour que la Temp soit 0 sur l'ocean plus profond que 600 m:
!      c'est le bon critere??:
!where (BSOC(:,:).lt.B(:,:)) Tann(:,:)=0.
where (sealevel-BSOC(:,:).gt.600) Tann(:,:)=0. 


Ts(:,:)=Tann(:,:)

time_change=1.e6
deltaTann=0.
ratiobil=1.

write(42,*)'Ces valeurs sont gardee jusqu a',time_change,'ans'
write(42,*)'apres : T=T+',deltaTann,'        bilan=bilan*',ratiobil

!jalv Acc(:,:)=bm(:,:)

! calcul de la fusion basale :
! premier essai : fusion basale cte :
!where (flot(:,:)) bmelt(:,:)=0.30

return
end subroutine input_clim 

!> SUBROUTINE: init_forclim
!! Suroutine pour la coherence des modules
!<
subroutine init_forclim
! on ne fait rien !
end subroutine init_forclim


!__________________________________________________________________________

!>SUBROUTINE: forclim
!!
!<
subroutine forclim
!cdc if (itracebug.eq.1)  call tracebug(' Heino: entree dans routine forclim')

if (time.gt.time_change) then
   Tann(:,:)=Tann(:,:)+deltaTann
   Ts(:,:)=Tann(:,:)
   bm(:,:)=bm(:,:)*ratiobil
endif


end subroutine forclim

end module climat_hudson
