!********************************************************************************
!                                                                                *
! module io_netcdf                                                                *
!                                                                                *
! NetCDF Fortran 90 read/write interface                                        *
! using input/output functions provided                                                *
! by unidata                                                                        *
!                                                                                *
!http://my.unidata.ucar.edu/content/software/netcdf/docs/netcdf-f90/index.html        *
!                                                                                *
!********************************************************************************
!
!
!
module io_netcdf
  !      

  use netcdf
  !      
  interface Read_Ncdf_var
     module procedure Read_Ncdf_var1d_Real,   &
          Read_Ncdf_var2d_Real,   &
          Read_Ncdf_var2d_Real_bis,   &
          Read_Ncdf_var3d_Real,   &
          Read_Ncdf_var4d_Real,   &
          Read_Ncdf_var3d_Real_t, &
          Read_Ncdf_var4d_Real_t, &
          Read_Ncdf_var4d_Real_nt, &
          Read_Ncdf_var1d_Int,    &
          Read_Ncdf_var2d_Int,    &
          Read_Ncdf_var3d_Int,    &
          Read_Ncdf_var4d_Int
  end interface
  !
  interface Write_Ncdf_var
     module procedure Write_Ncdf_var1d_Real,   &
          Write_Ncdf_var2d_Real,   &
          Write_Ncdf_var3d_Real,   &
          Write_Ncdf_var4d_Real,   &
          Write_Ncdf_var3d_Real_t, &
          Write_Ncdf_var4d_Real_t, &
          Write_Ncdf_var4d_Real_nt, &                           
          Write_Ncdf_var2d_Real_bis,&                           
          Write_Ncdf_var1d_Int,    &
          Write_Ncdf_var2d_Int,    &
          Write_Ncdf_var3d_Int,    &
          Write_Ncdf_var4d_Int
  end interface
  ! 
  interface Copy_Ncdf_att
     module procedure Copy_Ncdf_att_latlon,Copy_Ncdf_att_var
  end interface
  !
contains
  !
  !****************************************************************
  !   subroutine Read_Ncdf_dim                                    *
  !                                                               *
  ! subroutine to retrieve value of a given dimension             *
  !                                                               *
  !     dimname : name of dimension to retrieve                   *
  !     file    : netcdf file name                                *
  !     dimval  : value of the required dimension                 *
  !                                                               *
  !****************************************************************
  !
  subroutine Read_Ncdf_dim(dimname,file,dimval)   
    !      
    implicit none
    !           
    Character(*),intent(in) :: dimname,file    
    Integer    :: dimval
    !            
    ! local variables
    !
    integer ncid,status,dimid
    !     
    status = nf90_open(file,NF90_NOWRITE,ncid)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname,dimid)
    status = nf90_inquire_dimension(ncid,dimid,len=dimval)                                                                
    !      
    status = nf90_close(ncid)
    !      
  end subroutine Read_Ncdf_dim
  !
  !**************************************************************
  ! end subroutine Read_Ncdf_dim
  !**************************************************************      
  !
  !****************************************************************
  !   subroutine Write_Ncdf_dim                                   *
  !                                                               *
  ! subroutine to write a dimension in a given file               *
  !                                                               *
  !     dimname : name of dimension to initialize                 *
  !     file    : netcdf file name                                *
  !     dimval  : value of the dimension to write                 *
  !                                                               *
  !****************************************************************
  !
  subroutine Write_Ncdf_dim(dimname,file,dimval)   
    !      
    implicit none
    !           
    Character(*),intent(in) :: dimname,file    
    Integer    :: dimval
    !      
    ! local variables
    !
    integer ncid,status,dimid
    !     
    status = nf90_open(file,NF90_WRITE,ncid)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_redef(ncid)
    If(dimval.eq.0) then
       status = nf90_def_dim(ncid,dimname,nf90_unlimited,dimid)       
    Else
       status = nf90_def_dim(ncid,dimname,dimval,dimid)  
    End If

    status = nf90_enddef(ncid)
    !      

    status = nf90_close(ncid)
    !      
  end subroutine Write_Ncdf_dim
  !
  !**************************************************************
  ! end subroutine Write_Ncdf_dim
  !**************************************************************            
  ! 
  !****************************************************************
  !   subroutine Read_Ncdf_var                                    *
  !                                                               *
  ! subroutine to retrieve values of a given variable             *
  !                                                               *
  !     varname : name of variable to retrieve                    *
  !     file    : netcdf file name                                *
  !     tabvar  : array containing values of the required variable*
  !                                                               *
  !****************************************************************
  !          
  !**************************************************************
  !   subroutine Read_Ncdf_var1d_real
  !**************************************************************          
  !      
  subroutine Read_Ncdf_var1d_Real(varname,file,tabvar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Real*8, dimension(:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(1) :: dimID
    Integer :: dim1
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    status=nf90_inquire_variable(ncid,varid,dimids=dimID)
    status=nf90_inquire_dimension(ncid,dimID(1),len=dim1)
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1))  
    else
       if( any(shape(tabvar)/=(/dim1/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1))      
       endif
    endif
    !        
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Read_Ncdf_var1d_Real
  !           
  !**************************************************************
  ! end subroutine Read_Ncdf_var1d_real
  !**************************************************************          
  !            
  !           
  !**************************************************************
  !   subroutine Read_Ncdf_var2d_real
  !**************************************************************          
  !      
  subroutine Read_Ncdf_var2d_Real(varname,file,tabvar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Real*8, dimension(:,:), pointer :: tabvar
    !local variables
    Integer, dimension(10) :: dimIDS
    Integer :: dim1,dim2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2))  
    else
       if( any(shape(tabvar)/=(/dim1,dim2/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2))      
       endif
    endif
    !       
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var2d_Real
  !           
  !**************************************************************
  ! end subroutine Read_Ncdf_var2d_real
  !**************************************************************          
  !    

  !           
  !**************************************************************
  !   subroutine Read_Ncdf_var2d_real_bis
  !**************************************************************          
  !      
  subroutine Read_Ncdf_var2d_Real_bis(varname,file,tabvar,strt,cnt)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Real*8, dimension(:,:), pointer :: tabvar
    !local variables
    Integer, dimension(10) :: dimIDS
    Integer, dimension(2) :: strt,cnt
    Integer :: dim1,dim2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    dim1 = cnt(1) 
    dim2 = cnt(2)
    !      
    status = nf90_inq_varid(ncid,varname,varid)
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2))  
    else
       if( any(shape(tabvar)/=(/dim1,dim2/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2))      
       endif
    endif
    !       
    status=nf90_get_var(ncid,varid,tabvar,start = strt,count = cnt)     
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var2d_Real_bis
  !           
  !**************************************************************
  ! end subroutine Read_Ncdf_var2d_real_bis
  !**************************************************************          
  !  


  !           
  !**************************************************************
  !   subroutine Read_Ncdf_var3d_real
  !**************************************************************          
  !      
  subroutine Read_Ncdf_var3d_Real(varname,file,tabvar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Real*8, dimension(:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(10) :: dimIDS
    Integer :: dim1,dim2,dim3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)  
    status=nf90_inquire_dimension(ncid,dimIDS(3),len=dim3)
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2,dim3))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2,dim3/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2,dim3))      
       endif
    endif
    !      
    status=nf90_get_var(ncid,varid,tabvar) 
    if (status/=nf90_noerr) then    
       write(*,*)"unable to retrieve netcdf variable : ",trim(varname)
       stop
    endif
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var3d_Real
  !           
  !**************************************************************
  ! end subroutine Read_Ncdf_var3d_real
  !**************************************************************          
  ! 
  !           
  !**************************************************************
  !   subroutine Read_Ncdf_var4d_real
  !**************************************************************          
  !      
  subroutine Read_Ncdf_var4d_Real(varname,file,tabvar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Real*8, dimension(:,:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(10) :: dimIDS
    Integer :: dim1,dim2,dim3,dim4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)  
    status=nf90_inquire_dimension(ncid,dimIDS(3),len=dim3)   
    status=nf90_inquire_dimension(ncid,dimIDS(4),len=dim4)       
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2,dim3,dim4))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2,dim3,dim4/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2,dim3,dim4))      
       endif
    endif
    !  
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var4d_Real
  !           
  !**************************************************************
  ! end subroutine Read_Ncdf_var4d_real
  !**************************************************************          
  !             
  !           
  !**************************************************************
  !   subroutine Read_Ncdf_var1d_int
  !**************************************************************          
  !      
  subroutine Read_Ncdf_var1d_Int(varname,file,tabvar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer, dimension(:), pointer :: tabvar
    !
    !local variables
    !
    Integer,dimension(10) :: dimID
    Integer :: dim1
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    status=nf90_inquire_variable(ncid,varid,dimids=dimID)
    status=nf90_inquire_dimension(ncid,dimID(1),len=dim1)          
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1))  
    else
       if( any(shape(tabvar) /= (/dim1/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1))      
       endif
    endif
    !  
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    !      
  end subroutine Read_Ncdf_var1d_Int
  !           
  !**************************************************************
  ! end subroutine Read_Ncdf_var1d_int
  !**************************************************************          
  !            
  !           
  !**************************************************************
  !   subroutine Read_Ncdf_var2d_int
  !**************************************************************          
  !      
  subroutine Read_Ncdf_var2d_Int(varname,file,tabvar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer, dimension(:,:), pointer :: tabvar
    !local variables
    Integer, dimension(10) :: dimIDS
    Integer :: dim1,dim2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)      
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2))      
       endif
    endif
    !  
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    !      
  end subroutine Read_Ncdf_var2d_Int
  !           
  !**************************************************************
  ! end subroutine Read_Ncdf_var2d_Int
  !**************************************************************          
  !    

  !           
  !**************************************************************
  !   subroutine Read_Ncdf_var3d_Int
  !**************************************************************          
  !      
  subroutine Read_Ncdf_var3d_Int(varname,file,tabvar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer, dimension(:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(10) :: dimIDS
    Integer :: dim1,dim2,dim3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)  
    status=nf90_inquire_dimension(ncid,dimIDS(3),len=dim3)          
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2,dim3))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2,dim3/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2,dim3))      
       endif
    endif
    ! 
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    !      
  end subroutine Read_Ncdf_var3d_Int
  !           
  !**************************************************************
  ! end subroutine Read_Ncdf_var3d_int
  !**************************************************************  
  !
  !
  !
  !**************************************************************
  !   subroutine Read_Ncdf_var4d_int
  !**************************************************************          
  !      
  subroutine Read_Ncdf_var4d_Int(varname,file,tabvar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer, dimension(:,:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(10) :: dimIDS
    Integer :: dim1,dim2,dim3,dim4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)  
    status=nf90_inquire_dimension(ncid,dimIDS(3),len=dim3)   
    status=nf90_inquire_dimension(ncid,dimIDS(4),len=dim4)       
    !                
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2,dim3,dim4))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2,dim3,dim4/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2,dim3,dim4))      
       endif
    endif
    !  
    status=nf90_get_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var4d_Int
  !           
  !**************************************************************
  ! end subroutine Read_Ncdf_var4d_real
  !**************************************************************         
  !    
  !
  !****************************************************************
  !   subroutine Write_Ncdf_var                                   *
  !                                                               *
  ! subroutine to write a variable in a given file                *
  !                                                               *
  !     varname : name of variable to store                       *     
  !     dimname : name of dimensions of the given variable        *
  !     file    : netcdf file name                                *
  !     tabvar  : values of the variable to write                 *
  !                                                               *
  !****************************************************************
  !    
  !      
  !**************************************************************
  !   subroutine Write_Ncdf_var1d_real
  !**************************************************************    
  !      
  subroutine Write_Ncdf_var1d_Real(varname,dimname,file,tabvar,typevar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,dimname,typevar
    Real*8, dimension(:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname, dimid)
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)
    select case(TRIM(typevar))
    case('double')
       status = nf90_def_var(ncid,varname,nf90_double,(/dimid/),varid)
    case('float')
       status = nf90_def_var(ncid,varname,nf90_float,(/dimid/),varid)      
    end select
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var1d_Real
  !      
  !**************************************************************
  !   end subroutine Write_Ncdf_var1d_real
  !**************************************************************
  !      
  !**************************************************************
  !   subroutine Write_Ncdf_var2d_real
  !**************************************************************    
  !      
  subroutine Write_Ncdf_var2d_Real_bis(varname,dimname,file,tabvar,nbdim,typevar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Integer,intent(in) :: nbdim
    Character(*), dimension(4) :: dimname
    Real*8, dimension(:,:) :: tabvar
    Real*8, dimension(:,:,:),pointer :: tabtemp3d
    Real*8, dimension(:,:,:,:),pointer :: tabtemp4d
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3,dimid4
    Integer :: status,ncid
    Integer :: varid             
    ! 
    if(nbdim==4) then      
       Allocate(tabtemp4d(size(tabvar,1),size(tabvar,2),1,1))
       tabtemp4d(:,:,1,1) = tabvar(:,:)
    else if(nbdim==3) then
       Allocate(tabtemp3d(size(tabvar,1),size(tabvar,2),1))
       tabtemp3d(:,:,1) = tabvar(:,:)
    end if
    !      
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname(1), dimid1)
    status = nf90_inq_dimid(ncid,dimname(2), dimid2)
    status = nf90_inq_dimid(ncid,dimname(3), dimid3)
    !      
    if(nbdim==4) status = nf90_inq_dimid(ncid,dimname(4), dimid4)
    !      
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)
    if(nbdim==4 .and. typevar == 'double') then
       status = nf90_def_var(ncid,varname,nf90_double,     &
            (/dimid1,dimid2,dimid3,dimid4/),varid)
       !                              
    else if(nbdim==4 .and. typevar == 'float') then
       status = nf90_def_var(ncid,varname,nf90_float,     &
            (/dimid1,dimid2,dimid3,dimid4/),varid)                        
       !          
    else if(nbdim==3 .and. typevar == 'float') then
       status = nf90_def_var(ncid,varname,nf90_float,     &
            (/dimid1,dimid2,dimid3/),varid)
       !
    else if(nbdim==3 .and. typevar == 'double') then
       status = nf90_def_var(ncid,varname,nf90_double,     &
            (/dimid1,dimid2,dimid3/),varid)
       !                                                                 
    endif
    !                                    
    status = nf90_enddef(ncid)
    if(nbdim==4) status = nf90_put_var(ncid,varid,tabtemp4d)
    if(nbdim==3) status = nf90_put_var(ncid,varid,tabtemp3d)
    !      
    if(associated( tabtemp3d ) ) deallocate( tabtemp3d )   
    if(associated( tabtemp4d ) ) deallocate( tabtemp4d )     
    !
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var2d_Real_bis
  !      
  !**************************************************************
  !   end subroutine Write_Ncdf_var2d_real
  !**************************************************************  
  !   
  !      
  !**************************************************************
  !   subroutine Write_Ncdf_var2d_real
  !**************************************************************    
  !      
  subroutine Write_Ncdf_var2d_Real(varname,dimname,file,tabvar,typevar)
    !      
    !      implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*), dimension(2) :: dimname
    Real*8, dimension(:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname(1), dimid1)
    status = nf90_inq_dimid(ncid,dimname(2), dimid2)
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)

    select case(TRIM(typevar))
    case('double')
       status = nf90_def_var(ncid,varname,nf90_double,     &
            (/dimid1,dimid2/),varid)
    case('float')
       status = nf90_def_var(ncid,varname,nf90_float,     &
            (/dimid1,dimid2/),varid)     
    end select
    !
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var2d_Real
  !      
  !**************************************************************
  !   end subroutine Write_Ncdf_var2d_real
  !**************************************************************  
  !
  !      
  !**************************************************************
  !   subroutine Write_Ncdf_var3d_real
  !**************************************************************
  !      
  subroutine Write_Ncdf_var3d_Real(varname,dimname,file,tabvar,typevar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(3),intent(in) :: dimname
    Real*8, dimension(:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname(1), dimid1)
    status = nf90_inq_dimid(ncid,dimname(2), dimid2)
    status = nf90_inq_dimid(ncid,dimname(3), dimid3)
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)

    select case(TRIM(typevar))
    case('double')
       status = nf90_def_var(ncid,varname,nf90_double,     &
            (/dimid1,dimid2,dimid3/),varid)
    case('float')
       status = nf90_def_var(ncid,varname,nf90_float,     &
            (/dimid1,dimid2,dimid3/),varid)    
    end select
    !
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var3d_Real
  !      
  !**************************************************************
  !   end subroutine Write_Ncdf_var3d_real
  !**************************************************************
  ! 
  !
  !**************************************************************
  !   subroutine Write_Ncdf_var4d_real
  !**************************************************************
  !      
  subroutine Write_Ncdf_var4d_Real(varname,dimname,file,tabvar,typevar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(4),intent(in) :: dimname
    Real*8, dimension(:,:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3,dimid4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !
    status = nf90_inq_varid(ncid,varname,varid)
    !      
    if(status/=nf90_noerr) then
       !     
       status = nf90_inq_dimid(ncid,dimname(1), dimid1)
       status = nf90_inq_dimid(ncid,dimname(2), dimid2)
       status = nf90_inq_dimid(ncid,dimname(3), dimid3)
       status = nf90_inq_dimid(ncid,dimname(4), dimid4)
       status = nf90_redef(ncid)
       !      
       select case(TRIM(typevar))
       case('double')
          status = nf90_def_var(ncid,varname,nf90_double,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)
       case('float')
          status = nf90_def_var(ncid,varname,nf90_float,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)   
       end select
       !
       status = nf90_enddef(ncid)
    endif
    !         
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var4d_Real
  !      
  !**************************************************************
  !   end subroutine Write_Ncdf_var4d_real
  !**************************************************************
  !
  !
  !  
  !**************************************************************
  !   subroutine Write_Ncdf_var1d_Int
  !**************************************************************    
  !      
  subroutine Write_Ncdf_var1d_Int(varname,dimname,file,tabvar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,dimname
    Integer, dimension(:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid
    Integer :: status,ncid
    Integer :: varid             
    !
    !           print *,'ici tabvar = ',tabvar,varname,dimname
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname, dimid)
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)
    status = nf90_def_var(ncid,varname,nf90_int,(/dimid/),varid)
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var1d_Int
  !      
  !**************************************************************
  !   end subroutine Write_Ncdf_var1d_Int
  !**************************************************************
  !  
  !      
  !**************************************************************
  !   subroutine Write_Ncdf_var2d_Int
  !**************************************************************    
  !      
  subroutine Write_Ncdf_var2d_Int(varname,dimname,file,tabvar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Character(*), dimension(2) :: dimname
    Integer, dimension(:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname(1), dimid1)
    status = nf90_inq_dimid(ncid,dimname(2), dimid2)
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)
    status = nf90_def_var(ncid,varname,nf90_int,     &
         (/dimid1,dimid2/),varid)
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var2d_Int
  !      
  !**************************************************************
  !   end subroutine Write_Ncdf_var2d_Int
  !**************************************************************  
  ! 
  !      
  !**************************************************************
  !   subroutine Write_Ncdf_var3d_Int
  !**************************************************************
  !      
  subroutine Write_Ncdf_var3d_Int(varname,dimname,file,tabvar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Character(*),dimension(3),intent(in) :: dimname
    Integer, dimension(:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname(1), dimid1)
    status = nf90_inq_dimid(ncid,dimname(2), dimid2)
    status = nf90_inq_dimid(ncid,dimname(3), dimid3)
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)
    status = nf90_def_var(ncid,varname,nf90_int,     &
         (/dimid1,dimid2,dimid3/),varid)
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var3d_Int
  !      
  !**************************************************************
  !   end subroutine Write_Ncdf_var3d_Int
  !**************************************************************
  ! 
  !
  !**************************************************************
  !   subroutine Write_Ncdf_var4d_Int
  !**************************************************************
  !      
  subroutine Write_Ncdf_var4d_Int(varname,dimname,file,tabvar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Character(*),dimension(4),intent(in) :: dimname
    Integer, dimension(:,:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3,dimid4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_dimid(ncid,dimname(1), dimid1)
    status = nf90_inq_dimid(ncid,dimname(2), dimid2)
    status = nf90_inq_dimid(ncid,dimname(3), dimid3)
    status = nf90_inq_dimid(ncid,dimname(4), dimid4)
    status = nf90_inq_varid(ncid,varname,varid)
    status = nf90_redef(ncid)
    status = nf90_def_var(ncid,varname,nf90_int,     &
         (/dimid1,dimid2,dimid3,dimid4/),varid)
    status = nf90_enddef(ncid)
    status = nf90_put_var(ncid,varid,tabvar)     
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var4d_Int
  !      
  !**************************************************************
  !   end subroutine Write_Ncdf_var4d_Int
  !************************************************************** 
  !
  !
  !****************************************************************
  !   subroutine Read_Ncdf_var_t                                  *
  !                                                               *
  ! subroutine to read a variable in a given file for time t      *
  !                                                               *
  !     varname : name of variable to read                        *     
  !     file    : netcdf file name                                *
  !     tabvar  : values of the read variable                     *
  !     time    : time corresponding to the values to read        *        
  !                                                               *
  !****************************************************************
  !
  !
  !**************************************************************
  !   subroutine Read_Ncdf_var3d_real_t
  !**************************************************************          
  !      
  subroutine Read_Ncdf_var3d_Real_t(varname,file,tabvar,time)      
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer,intent(in) :: time
    Real*8, dimension(:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(3) :: dimIDS
    Integer :: dim1,dim2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    !        
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)
    !
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2,1))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2,1/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2,1))      
       endif
    endif

    status=nf90_get_var(ncid,varid,tabvar,start=(/1,1,time/))

    if (status/=nf90_noerr) then    
       write(*,*)"unable to retrieve netcdf variable : ",trim(varname)
       stop
    endif
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var3d_Real_t
  !           
  !**************************************************************
  ! end subroutine Read_Ncdf_var3d_real_t
  !**************************************************************          
  !
  !
  !
  !****************************************************************
  !   subroutine Write_Ncdf_var_t                                 *
  !                                                               *
  ! subroutine to write a variable in a given file for time t     *
  !                                                               *
  !     varname : name of variable to store                       *     
  !     dimname : name of dimensions of the given variable        *
  !     file    : netcdf file name                                *
  !     tabvar  : values of the variable to write                 *
  !     time    : time corresponding to the values to store       *        
  !                                                               *
  !****************************************************************
  !
  !      
  !**************************************************************
  !   subroutine Write_Ncdf_var3d_real_t
  !**************************************************************
  !      
  subroutine Write_Ncdf_var3d_Real_t(varname,dimname,file,tabvar,time,typevar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(3),intent(in) :: dimname
    Integer :: time
    Real*8, dimension(:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    if(time==1) then
       !      
       status = nf90_inq_dimid(ncid,dimname(1), dimid1)
       status = nf90_inq_dimid(ncid,dimname(2), dimid2)
       status = nf90_inq_dimid(ncid,dimname(3), dimid3)
       status = nf90_redef(ncid)

       !      
       select case(TRIM(typevar))
       case('double')
          status = nf90_def_var(ncid,varname,nf90_double,     &
               (/dimid1,dimid2,dimid3/),varid) 
       case('float')
          status = nf90_def_var(ncid,varname,nf90_float,     &
               (/dimid1,dimid2,dimid3/),varid)    
       end select
       !
       status = nf90_enddef(ncid)

    else
       status = nf90_inq_varid(ncid, varname, varid)
    endif
    !      
    status = nf90_put_var(ncid,varid,tabvar,start=(/1,1,time/))
    if (status/=nf90_noerr) then    
       write(*,*)"unable to store variable ",varname, &
            " in file ",file
       stop
    endif
    !     
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var3d_Real_t
  !      
  !**************************************************************
  !   end subroutine Write_Ncdf_var3d_real_t
  !**************************************************************
  !
  !
  !****************************************************************
  !   subroutine Read_Ncdf_var_t                                  *
  !                                                               *
  ! subroutine to read a variable in a given file for time t      *
  !                                                at level n     *
  !     varname : name of variable to read                        *     
  !     file    : netcdf file name                                *
  !     tabvar  : values of the read variable                     *
  !     time    : time corresponding to the values to read        *        
  !     level   : level corresponding to the values to read       *
  !                                                               *
  !****************************************************************
  !
  !
  !**************************************************************
  !   subroutine Read_Ncdf_var4d_real_nt
  !**************************************************************          
  !      
  subroutine Read_Ncdf_var4d_Real_nt(varname,file,tabvar,time,level)  
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer,intent(in) :: time,level
    Real*8, dimension(:,:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(4) :: dimIDS
    Integer :: dim1,dim2
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    !        
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)
    !
    if(.not. associated(tabvar)) then
       Allocate(tabvar(dim1,dim2,1,1))  
    else
       if( any(shape(tabvar) /= (/dim1,dim2,1,1/)) ) then           
          deallocate(tabvar)   
          Allocate(tabvar(dim1,dim2,1,1))      
       endif
    endif
    !
    status=nf90_get_var(ncid,varid,tabvar,start=(/1,1,level,time/))
    !                                             
    if (status/=nf90_noerr) then    
       write(*,*)"unable to retrieve netcdf variable : ",trim(varname)
       stop
    endif
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var4d_Real_nt
  !           
  !**************************************************************
  ! end subroutine Read_Ncdf_var4d_real_nt
  !**************************************************************          
  !
  !
  !**************************************************************
  !   subroutine Read_Ncdf_var4d_real_t
  !**************************************************************          
  !      
  subroutine Read_Ncdf_var4d_Real_t(varname,file,tabvar,time)     
    implicit none
    !       
    Character(*),intent(in) :: varname,file
    Integer,intent(in) :: time
    Real*8, dimension(:,:,:,:), pointer :: tabvar
    !
    !local variables
    !
    Integer, dimension(4) :: dimIDS
    Integer :: dim1,dim2,dim3
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_NOWRITE,ncid)
    !      
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !     
    status = nf90_inq_varid(ncid,varname,varid)
    !        
    status=nf90_inquire_variable(ncid,varid,dimids=dimIDS)
    status=nf90_inquire_dimension(ncid,dimIDS(1),len=dim1)
    status=nf90_inquire_dimension(ncid,dimIDS(2),len=dim2)
    status=nf90_inquire_dimension(ncid,dimIDS(3),len=dim3)
    !
    if(.not. associated(tabvar)) Allocate(tabvar(dim1,dim2,dim3,1))  
    status=nf90_get_var(ncid,varid,tabvar,start=(/1,1,1,time/))

    if (status/=nf90_noerr) then    
       write(*,*)"unable to retrieve netcdf variable : ",trim(varname)
       stop
    endif
    !     
    status = nf90_close(ncid)
    !     
  end subroutine Read_Ncdf_var4d_Real_t
  !           
  !**************************************************************
  ! end subroutine Read_Ncdf_var4d_real_t
  !**************************************************************          
  !
  !
  !****************************************************************
  !   subroutine Write_Ncdf_var_t                                 *
  !                                                               *
  ! subroutine to write a variable in a given file for time t     *
  !                                                at level n     *
  !     varname : name of variable to store                       *     
  !     dimname : name of dimensions of the given variable        *
  !     file    : netcdf file name                                *
  !     tabvar  : values of the variable to write                 *
  !     time    : time corresponding to the values to store       *        
  !     level   : level corresponding to the values to store      *
  !                                                               *
  !****************************************************************
  !
  !      
  !**************************************************************
  !   subroutine Write_Ncdf_var4d_real_t
  !**************************************************************
  !      
  subroutine Write_Ncdf_var4d_Real_t(varname,dimname,file,tabvar,time,typevar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(4),intent(in) :: dimname
    Integer :: time
    Real*8, dimension(:,:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3,dimid4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !      
    if(time==1) then
       !      
       status = nf90_inq_dimid(ncid,dimname(1), dimid1)
       status = nf90_inq_dimid(ncid,dimname(2), dimid2)
       status = nf90_inq_dimid(ncid,dimname(3), dimid3)
       status = nf90_inq_dimid(ncid,dimname(4), dimid4)
       status = nf90_redef(ncid)
       !      
       select case(TRIM(typevar))
       case('double')
          status = nf90_def_var(ncid,TRIM(varname),nf90_double,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)  
       case('float')
          status = nf90_def_var(ncid,TRIM(varname),nf90_float,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)    
       end select
       !
       status = nf90_enddef(ncid)

    else
       status = nf90_inq_varid(ncid, varname, varid)
    endif
    !    
    status = nf90_put_var(ncid,varid,tabvar,start=(/1,1,1,time/))
    if (status/=nf90_noerr) then    
       write(*,*)"unable to store variable ",varname, &
            " in file ",file
       stop
    endif
    !    
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var4d_Real_t
  !      
  !**************************************************************
  !   end subroutine Write_Ncdf_var4d_real_t
  !**************************************************************
  !
  !      
  !**************************************************************
  !   subroutine Write_Ncdf_var4d_real_nt
  !**************************************************************
  !      
  subroutine Write_Ncdf_var4d_Real_nt(varname,dimname,file,tabvar,time,level,typevar)
    !      
    implicit none
    !       
    Character(*),intent(in) :: varname,file,typevar
    Character(*),dimension(4),intent(in) :: dimname
    Integer :: time,level
    Real*8, dimension(:,:,:,:), pointer :: tabvar
    !
    ! local variables
    !
    Integer :: dimid1,dimid2,dimid3,dimid4
    Integer :: status,ncid
    Integer :: varid             
    !
    status = nf90_open(file,NF90_WRITE,ncid)       
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",file
       stop
    endif
    !      
    if(time==1.and.level==1) then
       !      
       status = nf90_inq_dimid(ncid,dimname(1), dimid1)
       status = nf90_inq_dimid(ncid,dimname(2), dimid2)
       status = nf90_inq_dimid(ncid,dimname(3), dimid3)
       status = nf90_inq_dimid(ncid,dimname(4), dimid4)
       status = nf90_redef(ncid)
       !      
       select case(TRIM(typevar))
       case('double')
          status = nf90_def_var(ncid,TRIM(varname),nf90_double,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)  
       case('float')
          status = nf90_def_var(ncid,TRIM(varname),nf90_float,     &
               (/dimid1,dimid2,dimid3,dimid4/),varid)    
       end select
       !
       status = nf90_enddef(ncid)

    else
       status = nf90_inq_varid(ncid, varname, varid)
    endif
    !    
    status = nf90_put_var(ncid,varid,tabvar,start=(/1,1,level,time/))
    if (status/=nf90_noerr) then    
       write(*,*)"unable to store variable ",varname, &
            " in file ",file
       stop
    endif
    !    
    status = nf90_close(ncid)
    ! 
  end subroutine Write_Ncdf_var4d_Real_nt
  !      
  !**************************************************************
  !   end subroutine Write_Ncdf_var4d_real_nt
  !**************************************************************
  !
  !****************************************************************
  !   subroutine Read_Ncdf_VarName                                *
  !                                                               *
  ! subroutine to retrieve of all variables                       *
  ! included in a given file                                      *
  !                                                               *
  !     filename    : netcdf file name                            *
  !     tabvarname  : array containing various variables names    *
  !                                                               *
  !****************************************************************
  !
  !**************************************************************
  !   subroutine Read_Ncdf_VarName
  !**************************************************************          
  !
  subroutine Read_Ncdf_VarName(filename,tabvarname)
    !      
    Character(*),intent(in) :: filename
    Character*20,dimension(:),pointer :: tabvarname
    Integer :: nDimensions,nVariables
    Integer :: nAttributes,unlimitedDimId,i
    Integer :: ncid,status
    !     
    status = nf90_open(filename,NF90_NOWRITE,ncid)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",filename
       stop
    endif
    !      
    status = nf90_inquire(ncid,nDimensions,nVariables,nAttributes, &
         unlimitedDimId) 
    !
    Allocate(tabvarname(nVariables))
    !
    Do i=1,nVariables
       status = nf90_inquire_variable(ncid,i,tabvarname(i))
    End do

  end subroutine Read_Ncdf_Varname
  !
  !**************************************************************
  !   end subroutine Read_Ncdf_VarName
  !**************************************************************          
  !
  !**************************************************************
  !   subroutine Copy_Ncdf_att
  !**************************************************************          
  !
  subroutine Copy_Ncdf_att_var(varname,filein,fileout)
    !      
    Character(*),intent(in) :: filein,fileout
    Character(*),intent(in) :: varname
    Integer :: ncid_in,ncid_out,status,varid_in,varid_out
    !     
    !      print *,'filein = ',filein,fileout
    status = nf90_open(filein,NF90_NOWRITE,ncid_in)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open input netcdf file : ",filein
       stop
    endif
    !                                                                             
    status = nf90_open(fileout,NF90_WRITE,ncid_out)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open output netcdf file : ",fileout
       stop
    endif
    !  
    !      print *,'ici1'
    status = nf90_inq_varid(ncid_in,varname,varid_in)
    status = nf90_inq_varid(ncid_out,varname,varid_out)
    !
    status = nf90_redef(ncid_out)
    !      
    status = nf90_copy_att(ncid_in,varid_in,'units',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'valid_min',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'valid_max',ncid_out,varid_out) 
    status = nf90_copy_att(ncid_in,varid_in,'long_name',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'calendar',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'title',ncid_out,varid_out)     
    status = nf90_copy_att(ncid_in,varid_in,'time_origin',ncid_out,varid_out) 
    status = nf90_copy_att(ncid_in,varid_in,'positive',ncid_out,varid_out)                 
    status = nf90_copy_att(ncid_in,varid_in,'tstep_sec',ncid_out,varid_out)          
    status = nf90_copy_att(ncid_in,varid_in,'nav_model',ncid_out,varid_out)   
    status = nf90_copy_att(ncid_in,varid_in,'Minvalue=',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'Maxvalue=',ncid_out,varid_out)  
    status = nf90_copy_att(ncid_in,varid_in,'short_name',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'online_operation',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'axis',ncid_out,varid_out)            
    status = nf90_copy_att(ncid_in,varid_in,'interval_operation',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'interval_write',ncid_out,varid_out) 
    status = nf90_copy_att(ncid_in,varid_in,'associate',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'actual_range',ncid_out,varid_out) 
    status = nf90_copy_att(ncid_in,varid_in,'longitude',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'latitude',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'scale_factor',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'add_offset',ncid_out,varid_out)
    status = nf90_copy_att(ncid_in,varid_in,'missing_value',ncid_out,varid_out) 
    !      
    status = nf90_enddef(ncid_out)  
    !
    status = nf90_close(ncid_in)
    status = nf90_close(ncid_out)
    !      print *,'ici2'
    ! 
  end subroutine Copy_Ncdf_att_var
  !
  !**************************************************************
  !   end subroutine Copy_Ncdf_att
  !**************************************************************         
  !**************************************************************
  !   subroutine Copy_Ncdf_att
  !**************************************************************          
  !
  subroutine Copy_Ncdf_att_latlon(varname,filein,fileout,min,max)
    !      
    Character(*),intent(in) :: filein,fileout
    Character(*),intent(in) :: varname
    real*8 :: min,max
    Integer :: ncid_in,ncid_out,status,varid_in,varid_out
    !     
    status = nf90_open(filein,NF90_NOWRITE,ncid_in)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",filein
       stop
    endif
    !                                                                             
    status = nf90_open(fileout,NF90_WRITE,ncid_out)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",fileout
       stop
    endif
    !  
    status = nf90_inq_varid(ncid_in,varname,varid_in)
    status = nf90_inq_varid(ncid_out,varname,varid_out)
    !
    status = nf90_redef(ncid_out)
    !      
    select case (varname)
       !      
    case('nav_lon')     
       status = nf90_copy_att(ncid_in,varid_in,'units',ncid_out,varid_out) 
       status = nf90_put_att(ncid_out,varid_out,'valid_min',real(min,4))
       status = nf90_put_att(ncid_out,varid_out,'valid_max',real(max,4))
       status = nf90_copy_att(ncid_in,varid_in,'long_name',ncid_out,varid_out)
       status = nf90_copy_att(ncid_in,varid_in,'nav_model',ncid_out,varid_out)
       status = nf90_copy_att(ncid_in,varid_in,'title',ncid_out,varid_out)
       !
    case('nav_lat')
       status = nf90_copy_att(ncid_in,varid_in,'units',ncid_out,varid_out) 
       status = nf90_put_att(ncid_out,varid_out,'valid_min',real(min,4))
       status = nf90_put_att(ncid_out,varid_out,'valid_max',real(max,4))
       status = nf90_copy_att(ncid_in,varid_in,'long_name',ncid_out,varid_out)
       status = nf90_copy_att(ncid_in,varid_in,'nav_model',ncid_out,varid_out)
       status = nf90_copy_att(ncid_in,varid_in,'title',ncid_out,varid_out) 
       !
    end select
    !      
    status = nf90_enddef(ncid_out)  
    !
    status = nf90_close(ncid_in)
    status = nf90_close(ncid_out)
  end subroutine Copy_Ncdf_att_latlon
  !*************************************************************
  !**************************************************************
  !**************************************************************
  !   function Get_NbDims
  !**************************************************************          
  !
  Integer function Get_NbDims( varname , filename )
    !
    Character(*),intent(in) :: varname,filename
    Integer :: status,ncid,varid    
    !      
    status = nf90_open(TRIM(filename),NF90_NOWRITE,ncid)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",TRIM(filename)
       stop
    endif
    status = nf90_inq_varid(ncid,TRIM(varname),varid)     
    status = nf90_inquire_variable(ncid, varid , ndims = Get_NbDims)
    !
    return
    !
  end function Get_NbDims
  !
  !*************************************************************
  !**************************************************************
  !   function Get_NbDims
  !**************************************************************          
  !
  Logical function Dims_Existence( dimname , filename )
    !
    Character(*),intent(in) :: dimname,filename
    Integer :: status,ncid,dimid    
    !      
    status = nf90_open(TRIM(filename),NF90_NOWRITE,ncid)
    if (status/=nf90_noerr) then    
       write(*,*)"unable to open netcdf file : ",TRIM(filename)
       stop
    endif
    status = nf90_inq_dimid(ncid,dimname,dimid)
    !      
    if (status/=nf90_noerr) then
       Dims_Existence = .false.
    else
       Dims_Existence = .true.
    endif
    !
    return
    !
  end function Dims_Existence
  !
  !*************************************************************
  !**************************************************************

end module io_netcdf
