!> \file resol_adv_diff_2D.F90
!! Module pour la resolution de l'equation d advection diffusion 2D
!<

!> \namespace reso_adv_diff_2D
!! Module pour la resolution de l'equation d advection diffusion 2D
!! \author ...
!! \date ...
!! @note Used module
!! @note   - use module3D_phy
!<

module reso_adv_diff_2D

use module3D_phy



implicit none
real :: omega  ! parametre schema temporel de la resolution partie diffusion
real :: mu_adv ! parametre schema temporel de la resolution advection

real :: upwind  ! schema spatial pour l'advection

integer, dimension(2) :: ijmax    ! position de maxval
integer :: iFAIL

contains 

subroutine init_reso_adv_diff_2D


write(num_rep_42,*)'partie diffusive'
write(num_rep_42,*)'le type de schema temporel diffusif depend de omega'
write(num_rep_42,*)'0 -> explicite, 0.5 -> Crank-Nicolson'
write(num_rep_42,*)'1 -> semi implicite,  >1 -> over-implicite'

      omega = 2.5

write(num_rep_42,*)'omega = ',omega

write(num_rep_42,*)
write(num_rep_42,*)'partie advective'
write(num_rep_42,*)' le schma temporel advectif dpend de mu'
write(num_rep_42,*)'0 -> explicite, 0.5 -> Crank-Nicolson'
write(num_rep_42,*)'1 -> semi implicite,  >1 -> over-implicite'

mu_adv=1.
write(num_rep_42,*)'mu_adv = ',mu_adv

write(num_rep_42,*)' le schma spatial dpend de upwind'
write(num_rep_42,*)'1 -> schema upwind,  0.5 -> schema centre'

upwind=1
write(num_rep_42,*)'upwind = ',upwind


write(num_rep_42,*)'------------------------------------------------------'

return
end subroutine init_reso_adv_diff_2D
!------------------------------------------------------------------

subroutine resol_adv_diff_2D (Dfx,Dfy,advx,advy,vieuxH)

  implicit none

  real,dimension(nx,ny), intent(in) :: Dfx      ! terme diffusif selon x
  real,dimension(nx,ny), intent(in) :: Dfy      ! terme diffusif selon y
  real,dimension(nx,ny), intent(in) :: Advx    ! terme advectif selon x
  real,dimension(nx,ny), intent(in) :: Advy    ! terme advectif selon x
  real,dimension(nx,ny), intent(in) :: vieuxH  ! terme advectif selon x


  ! tableaux de travail. resolution M H = Frelax
  real,dimension(nx,ny) :: crelax      ! diagnonale de M
  real,dimension(nx,ny) :: arelax      ! sous diagonale selon x
  real,dimension(nx,ny) :: brelax      ! sur diagonale selon x
  real,dimension(nx,ny) :: drelax      ! sous diagonale selon y
  real,dimension(nx,ny) :: erelax      ! sur diagonale selon y
  real,dimension(nx,ny) :: frelax      ! vecteur
  real,dimension(nx,ny) :: c_west      ! sur demi mailles Ux
  real,dimension(nx,ny) :: c_east      ! sur demi mailles Ux
  real,dimension(nx,ny) :: c_north     ! sur demi mailles Uy
  real,dimension(nx,ny) :: c_south     !sur demi mailles Ux

  real,dimension(nx,ny) :: bdx         ! pente socle
  real,dimension(nx,ny) :: bdy         ! pente socle

  real,dimension(nx,ny) :: hdx         ! pente epaisseur
  real,dimension(nx,ny) :: hdy         ! pente epaisseur

  real :: frdx,frdy                    ! pour calcul frelax : termes diffusion
  real :: fraxw,fraxe,frays,frayn      ! termes advection

  real,dimension(nx,ny) :: deltah      ! dans calcul relax
  real :: delh                         ! dans calcul relax
  real :: testh                        ! dans calcul relax

  logical :: stopp
  integer :: ntour
  real :: reste


  integer :: it1,it2,jt1,jt2           ! pour des tests d'asymtrie

  ! attention H et bm sont passs par le module  module3D_phy

  if (itracebug.eq.1)  call tracebug(' Entree dans routine resolution_diffusion')

  ! calcul de bdx et hdx
  hdx(:,:)=dx1*(vieuxH(:,:)-eoshift(vieuxH(:,:),shift=-1,boundary=0.,dim=1))
  hdy(:,:)=dx1*(vieuxH(:,:)-eoshift(vieuxH(:,:),shift=-1,boundary=0.,dim=2))
  bdx(:,:)=dx1*(B(:,:)-eoshift(B(:,:),shift=-1,boundary=0.,dim=1))
  bdy(:,:)=dx1*(B(:,:)-eoshift(B(:,:),shift=-1,boundary=0.,dim=2))


  ! initialisations (qui feront aussi les conditions aux limites)
  Arelax(:,:)=0.
  Brelax(:,:)=0.
  Drelax(:,:)=0.
  Erelax(:,:)=0.
  Crelax(:,:)=1.
  Frelax(:,:)=0.
  DeltaH(:,:)=0.

  H(1,:)=0.  ! bord gauche
  H(nx,:)=0. ! bord droit
  H(:,1)=0.  ! bord bas
  H(:,ny)=0.  ! bord haut


  ! schema spatial

  if (upwind.eq.1.) then                 !schema amont

     where (Advx(:,:).ge.0.)
        c_west(:,:)=1.
        c_east(:,:)=0.
     elsewhere
        c_west(:,:)=0.
        c_east(:,:)=1.
     end where

     where (Advy(:,:).ge.0.)
        c_south(:,:)=1.
        c_north(:,:)=0.
     elsewhere
        c_south(:,:)=0.
        c_north(:,:)=1.
     end where

  else if (upwind.lt.1.) then             ! schema centre
     c_west(:,:)=0.5
     c_east(:,:)=0.5
     c_south(:,:)=0.5
     c_north(:,:)=0.5
  end if

  ! attribution des elements des diagonales
  do j=2,ny-1
     do i=2,nx-1

        !  sous diagonale en x
        arelax(i,j)=-omega*Dtdx2*Dfx(i,j)   &               ! partie diffusive en x
             -mu_adv*dtdx*c_west(i,j)*Advx(i,j)            ! partie advective en x

        !  sur diagonale en x
        brelax(i,j)=-omega*Dtdx2*Dfx(i+1,j)  &              ! partie diffusive
             +mu_adv*dtdx*c_east(i+1,j)*Advx(i+1,j)        ! partie advective

        !  sous diagonale en y
        drelax(i,j)=-omega*Dtdx2*Dfy(i,j)   &               ! partie diffusive en y
             -mu_adv*dtdx*c_south(i,j)*Advy(i,j)           ! partie advective en y

        !  sur diagonale en y
        erelax(i,j)=-omega*Dtdx2*Dfy(i,j+1)  &              ! partie diffusive
             +mu_adv*dtdx*c_north(i,j+1)*Advy(i,j+1)       ! partie advective



        ! diagonale
        crelax(i,j)=omega*Dtdx2*((Dfx(i+1,j)+Dfx(i,j))  &                         ! partie diffusive en x
             +(Dfy(i,j+1)+Dfy(i,j)))                           ! partie diffusive en y
        crelax(i,j)=crelax(i,j)+mu_adv*dtdx* &
             ( (c_west(i+1,j)*Advx(i+1,j)-c_east(i,j)*Advx(i,j)) &      !partie advective en x
             +(c_south(i,j+1)*Advy(i,j+1)-c_north(i,j)*Advy(i,j)))      !partie advective en y
        crelax(i,j)=1.+crelax(i,j)                                              ! partie temporelle


        ! terme du vecteur

        frdx= -Dfx(i,j) * (Bdx(i,j)  +(1.-omega)*Hdx(i,j))         &  ! partie diffusive en x
             +Dfx(i+1,j)*(Bdx(i+1,j)+(1.-omega)*Hdx(i+1,j))

        frdy= -Dfy(i,j) * (Bdy(i,j)  +(1.-omega)*Hdy(i,j))         &  ! partie diffusive en y
             +Dfy(i,j+1)*(Bdy(i,j+1)+(1.-omega)*Hdy(i,j+1))

        fraxw= -c_west(i,j)*  Advx(i,j) * vieuxH(i-1,j)           &  ! partie advective en x 
             +c_west(i+1,j)*Advx(i+1,j)*vieuxH(i,j)                ! venant de l'west

        fraxe= -c_east(i,j) * Advx(i,j) * vieuxH(i,j)             &  ! partie advective en x 
             +c_east(i+1,j)*Advx(i+1,j)*vieuxH(i+1,j)              ! venant de l'est

        frays= -c_south(i,j) * Advy(i,j) * vieuxH(i,j-1)          &  ! partie advective en y
             +c_south(i,j+1)*Advy(i,j+1)*vieuxH(i,j)               ! venant du sud

        frayn= -c_north(i,j) * Advy(i,j) * vieuxH(i,j)            &  ! partie advective en y
             +c_north(i,j+1)*Advy(i,j+1)*vieuxH(i,j+1)             ! venant du nord




        frelax(i,j)=vieuxH(i,j)+Dt*(bm(i,j)-bmelt(i,j))+dtdx*(frdx+frdy) &
             + (1.-mu_adv)*dtdx*((fraxw+fraxe)+(frays+frayn))


     end do
  end do

  debug_3D(:,:,44)=bm(:,:)-bmelt(:,:)


  ! Boucle de relaxation :
  ! ----------------------

!!$write(166,*)'time,diagonale', time,dt,maxval(crelax(:,:)),maxloc(crelax)
!!$write(166,*)'autres',maxval(abs(arelax(:,:))+abs(brelax(:,:))+abs(drelax(:,:))+abs(erelax(:,:))), &
!!$maxloc(abs(arelax(:,:))+abs(brelax(:,:))+abs(drelax(:,:))+abs(erelax(:,:)))

!!$
!!$if (time.gt.22100.) then
!!$ijmax(:)=maxloc(abs(arelax(:,:))+abs(brelax(:,:))+abs(drelax(:,:))+abs(erelax(:,:)))
!!$
!!$write(166,*)'arelax',arelax(ijmax(1),ijmax(2)),-omega*Dtdx2*dfx(ijmax(1),ijmax(2)),-mu_adv*dtdx*advx(ijmax(1),ijmax(2))
!!$write(166,*)'brelax',brelax(ijmax(1),ijmax(2)),-omega*Dtdx2*dfx(ijmax(1)+1,ijmax(2)),-mu_adv*dtdx*advx(ijmax(1)+1,ijmax(2))
!!$write(166,*)'drelax',drelax(ijmax(1),ijmax(2)),-omega*Dtdx2*dfy(ijmax(1),ijmax(2)),-mu_adv*dtdx*advy(ijmax(1),ijmax(2))
!!$write(166,*)'erelax',erelax(ijmax(1),ijmax(2)),-omega*Dtdx2*dfy(ijmax(1),ijmax(2)+1),-mu_adv*dtdx*advy(ijmax(1),ijmax(2)+1)
!!$
!!$end if

  stopp = .false.
  ntour=0

!!$do j=2,ny-1
!!$   do i=2,nx-1
!!$      if (abs(crelax(i,j)).le.1.e-15) then
!!$         write(166,*) 'time ',time,i,j,crelax(i,j)
!!$         write(166,'(6(es12.5,1x))') advx(i-1,j),advx(i,j),advx(i+1,j),advy(i,j-1),advy(i,j),advy(i,j+1)
!!$         write(166,'(6(es12.5,1x))') dfx(i-1,j),dfx(i,j),dfx(i+1,j),dfy(i,j-1),dfy(i,j),dfy(i,j+1)
!!$      end if
!!$   end do
!!$end do

!!$write(166,*) 'dtdx=',dtdx,'dt=',dt
!!$write(166,*) 'arelax,brelax,crelax,drelax,erelax,frelax'
!!$
!!$do j=43,45
!!$write(166,*) 'j=',j
!!$   write(166,*) arelax(118:122,j)
!!$   write(166,*) brelax(118:122,j)
!!$   write(166,*) crelax(118:122,j)
!!$   write(166,*) drelax(118:122,j)
!!$   write(166,*) erelax(118:122,j)
!!$   write(166,*) frelax(118:122,j)
!!$
!!$end do


  relax_loop: do while(.not.stopp)
     ntour=ntour+1
!!$if (time.gt.22102.) then
!!$   write(166,*) time, ntour,testh,dt
!!$end if


     do j=2,ny-1
        do i=2,nx-1

!!$      reste = (((arelax(i,j)*H(i-1,j) + brelax(i,j)*H(i+1,j)) &
!!$           + (drelax(i,j)*H(i,j-1) + erelax(i,j)*H(i,j+1))) &
!!$           + crelax(i,j)*H(i,j))- frelax(i,j)

           reste = (((arelax(i,j)*H(i-1,j) +drelax(i,j)*H(i,j-1)) &
                + (brelax(i,j)*H(i+1,j) + erelax(i,j)*H(i,j+1))) &
                + crelax(i,j)*H(i,j))- frelax(i,j)

           if (ntour.eq.1)  debug_3D(i,j,49)=(((arelax(i,j)*H(i-1,j) +drelax(i,j)*H(i,j-1)) &
                + (brelax(i,j)*H(i+1,j) + erelax(i,j)*H(i,j+1))) &
                + crelax(i,j)*H(i,j))


           deltaH(i,j) = reste/crelax(i,j)

        end do
     end do

     debug_3D(:,:,50)=arelax(:,:)
     debug_3D(:,:,51)=brelax(:,:)
     debug_3D(:,:,52)=crelax(:,:)
     debug_3D(:,:,53)=drelax(:,:)
     debug_3D(:,:,54)=erelax(:,:)
     debug_3D(:,:,55)=frelax(:,:)




     H(:,:)=H(:,:)-deltaH(:,:)

!!$if (ntour.eq.1) then
!!$write(166,*) 'deltaH'
!!$
!!$do j=43,45
!!$write(166,*) 'j=',j
!!$   write(166,*) deltaH(118:122,j)
!!$end do
!!$
!!$end if

     ! critere d'arret:
     ! ----------------         

     delh=0


     do j=2,ny-1
        do i=2,nx-1
           delh=delh+deltaH(i,j)**2
        end do
     end do

!!$if (time.gt.22102.) then
!!$   write(166,*) delh,maxval(deltaH(:,:)),maxloc(deltaH(:,:))
!!$end if

     if (delh.gt.0.) then
        testh=sqrt(delh)/((nx-2)*(ny-2))
     else
        testh=0.
     endif
     stopp = (testh.lt.1.e-4).or.(ntour.gt.100)

     !write(166,*) 'sortie relax',time, ntour,testh,maxval(deltaH(:,:))
  end do relax_loop

  !write(166,*) time,testh,ntour

!!$! conditions aux limites
!!$

  ! tests d'asymtrie
!!$it1=2
!!$it2=80
!!$jt1=41
!!$jt2=41
!!$if (H(it1,jt1).ne.H(it2,jt2)) then
!!$   write(6,*)'asymetrie H time',time,'dans resol'
!!$   write(6,*) 'bm',bm(it1,jt1),bm(it2,jt2)
!!$   write(6,*) 'vieuxH',vieuxH(it1,jt1),vieuxH(it2,jt2)
!!$   write(6,*) 'H',H(it1,jt1),H(it2,jt2)
!!$   write(6,*) 'dfx',dfx(it1,jt1),dfx(it1+1,jt1),dfx(it2,jt2),dfx(it2+1,jt2)
!!$   write(6,*) 'diffmx',diffmx(it1,jt1),diffmx(it1+1,jt1),diffmx(it2,jt2),diffmx(it2+1,jt2)
!!$   write(6,*) 'uxbar',uxbar(it1,jt1),uxbar(it1+1,jt1),uxbar(it2,jt2),uxbar(it2+1,jt2)
!!$   write(6,*) 'dfy',dfy(it1,jt1),dfy(it2,jt2),dfy(it1,jt1+1),dfy(it2,jt2+1)
!!$   write(6,*) 'sdx',sdx(it1,jt1),sdx(it2,jt2),sdx(it1+1,jt1),sdx(it2-1,jt2)
!!$   write(6,*) 'sdy',sdy(it1,jt1),sdy(it2,jt2),sdy(it1,jt1+1),sdy(it2,jt2+1)
!!$   write(6,*) 'sdxmy',sdxmy(it1,jt1),sdxmy(it2,jt2),sdxmy(it1+1,jt1),sdxmy(it2+1,jt2)
!!$   write(6,*) 'sdymx',sdymx(it1,jt1),sdymx(it2,jt2),sdymx(it1,jt1+1),sdymx(it2,jt2+1)
!!$   write(6,*)'ddbx',ddbx(it1,jt1),ddby(it2,jt2)
!!$   write(6,*)'ddx(i,j,1)',ddx(it1,it2,1),ddy(it2,it1,1)
!!$   write(6,*) 'arelax',arelax(it1,jt1),arelax(it2,jt2),arelax(it1+1,jt1),arelax(it2-1,jt2)
!!$   write(6,*) 'brelax',brelax(it1,jt1),brelax(it2,jt2),brelax(it1+1,jt1),brelax(it2-1,jt2)
!!$   write(6,*) 'drelax',drelax(it1,jt1),drelax(it2,jt2)
!!$   write(6,*) 'erelax',erelax(it1,jt1),erelax(it2,jt2)
!!$   write(6,*) 'crelax',crelax(it1,jt1),crelax(it2,jt2)
!!$   write(6,*) 'frelax',frelax(it1,jt1),frelax(it2,jt2)
!!$   stop
!!$endif



  !   write(166,*) '---------------------------------------------------------------'



  return

end subroutine resol_adv_diff_2D
!_______________________________________________________________________________
!> In this version, H can be prescribed on some nodes
!! eventually changing with time
!! value H_presc et test I_Hpresc
subroutine resol_adv_diff_2D_Hpresc (Dfx,Dfy,advx,advy,vieuxH,H_presc,i_Hpresc)

implicit none

real,dimension(nx,ny), intent(in) :: Dfx          ! terme diffusif selon x
real,dimension(nx,ny), intent(in) :: Dfy          ! terme diffusif selon y
real,dimension(nx,ny), intent(in) :: Advx         ! terme advectif selon x
real,dimension(nx,ny), intent(in) :: Advy         ! terme advectif selon x
real,dimension(nx,ny), intent(in) :: vieuxH       ! terme advectif selon x

real,dimension(nx,ny), intent(in) :: H_presc      ! H value if prescribed
integer,dimension(nx,ny), intent(in) :: i_Hpresc  ! 1 if H is prescribed on this node, else 0


! tableaux de travail. resolution M H = Frelax
real,dimension(nx,ny) :: crelax      ! diagnonale de M
real,dimension(nx,ny) :: arelax      ! sous diagonale selon x
real,dimension(nx,ny) :: brelax      ! sur diagonale selon x
real,dimension(nx,ny) :: drelax      ! sous diagonale selon y
real,dimension(nx,ny) :: erelax      ! sur diagonale selon y
real,dimension(nx,ny) :: frelax      ! vecteur
real,dimension(nx,ny) :: c_west      ! sur demi mailles Ux
real,dimension(nx,ny) :: c_east      ! sur demi mailles Ux
real,dimension(nx,ny) :: c_north     ! sur demi mailles Uy
real,dimension(nx,ny) :: c_south     !sur demi mailles Ux

real,dimension(nx,ny) :: bdx         ! pente socle
real,dimension(nx,ny) :: bdy         ! pente socle

real,dimension(nx,ny) :: hdx         ! pente epaisseur
real,dimension(nx,ny) :: hdy         ! pente epaisseur

real :: frdx,frdy                    ! pour calcul frelax : termes diffusion
real :: fraxw,fraxe,frays,frayn      ! termes advection

real,dimension(nx,ny) :: deltah      ! dans calcul relax
real :: delh                         ! dans calcul relax
real :: testh                        ! dans calcul relax

logical :: stopp
integer :: ntour
real :: reste


integer :: it1,it2,jt1,jt2           ! pour des tests d'asym??trie

! attention H et bm sont pass??s par le module  module3D_phy

if (itracebug.eq.1)  call tracebug(' Entree dans routine resolution_diffusion')

! calcul de bdx et hdx
hdx(:,:)=dx1*(vieuxH(:,:)-eoshift(vieuxH(:,:),shift=-1,boundary=0.,dim=1))
hdy(:,:)=dx1*(vieuxH(:,:)-eoshift(vieuxH(:,:),shift=-1,boundary=0.,dim=2))
bdx(:,:)=dx1*(B(:,:)-eoshift(B(:,:),shift=-1,boundary=0.,dim=1))
bdy(:,:)=dx1*(B(:,:)-eoshift(B(:,:),shift=-1,boundary=0.,dim=2))


! initialisations (qui feront aussi les conditions aux limites)
Arelax(:,:)=0.
Brelax(:,:)=0.
Drelax(:,:)=0.
Erelax(:,:)=0.
Crelax(:,:)=1.
Frelax(:,:)=0.
DeltaH(:,:)=0.


H(1,:)=0.  ! bord gauche
H(nx,:)=0. ! bord droit
H(:,1)=0.  ! bord bas
H(:,ny)=0.  ! bord haut


! schema spatial

if (upwind.eq.1.) then                 !schema amont

   where (Advx(:,:).ge.0.)
      c_west(:,:)=1.
      c_east(:,:)=0.
   elsewhere
      c_west(:,:)=0.
      c_east(:,:)=1.
   end where

   where (Advy(:,:).ge.0.)
      c_south(:,:)=1.
      c_north(:,:)=0.
   elsewhere
      c_south(:,:)=0.
      c_north(:,:)=1.
   end where

else if (upwind.lt.1.) then             ! schema centre
      c_west(:,:)=0.5
      c_east(:,:)=0.5
      c_south(:,:)=0.5
      c_north(:,:)=0.5
end if

! attribution des elements des diagonales
do j=2,ny-1
  do i=2,nx-1

!  sous diagonale en x
     arelax(i,j)=-omega*Dtdx2*Dfx(i,j)   &               ! partie diffusive en x
          -mu_adv*dtdx*c_west(i,j)*Advx(i,j)            ! partie advective en x

!  sur diagonale en x
     brelax(i,j)=-omega*Dtdx2*Dfx(i+1,j)  &              ! partie diffusive
          +mu_adv*dtdx*c_east(i+1,j)*Advx(i+1,j)        ! partie advective

!  sous diagonale en y
     drelax(i,j)=-omega*Dtdx2*Dfy(i,j)   &               ! partie diffusive en y
          -mu_adv*dtdx*c_south(i,j)*Advy(i,j)           ! partie advective en y

!  sur diagonale en y
     erelax(i,j)=-omega*Dtdx2*Dfy(i,j+1)  &              ! partie diffusive
          +mu_adv*dtdx*c_north(i,j+1)*Advy(i,j+1)       ! partie advective



! diagonale
     crelax(i,j)=omega*Dtdx2*((Dfx(i+1,j)+Dfx(i,j))  &                         ! partie diffusive en x
                             +(Dfy(i,j+1)+Dfy(i,j)))                           ! partie diffusive en y
     crelax(i,j)=crelax(i,j)+mu_adv*dtdx* &
                  ( (c_west(i+1,j)*Advx(i+1,j)-c_east(i,j)*Advx(i,j)) &      !partie advective en x
                  +(c_south(i,j+1)*Advy(i,j+1)-c_north(i,j)*Advy(i,j)))      !partie advective en y
     crelax(i,j)=1.+crelax(i,j)                                              ! partie temporelle


! terme du vecteur

     frdx= -Dfx(i,j) * (Bdx(i,j)  +(1.-omega)*Hdx(i,j))         &  ! partie diffusive en x
           +Dfx(i+1,j)*(Bdx(i+1,j)+(1.-omega)*Hdx(i+1,j))

     frdy= -Dfy(i,j) * (Bdy(i,j)  +(1.-omega)*Hdy(i,j))         &  ! partie diffusive en y
           +Dfy(i,j+1)*(Bdy(i,j+1)+(1.-omega)*Hdy(i,j+1))

     fraxw= -c_west(i,j)*  Advx(i,j) * vieuxH(i-1,j)           &  ! partie advective en x 
            +c_west(i+1,j)*Advx(i+1,j)*vieuxH(i,j)                ! venant de l'west

     fraxe= -c_east(i,j) * Advx(i,j) * vieuxH(i,j)             &  ! partie advective en x 
            +c_east(i+1,j)*Advx(i+1,j)*vieuxH(i+1,j)              ! venant de l'est

     frays= -c_south(i,j) * Advy(i,j) * vieuxH(i,j-1)          &  ! partie advective en y
            +c_south(i,j+1)*Advy(i,j+1)*vieuxH(i,j)               ! venant du sud

     frayn= -c_north(i,j) * Advy(i,j) * vieuxH(i,j)            &  ! partie advective en y
            +c_north(i,j+1)*Advy(i,j+1)*vieuxH(i,j+1)             ! venant du nord




     frelax(i,j)=vieuxH(i,j)+Dt*(bm(i,j)-bmelt(i,j))+dtdx*(frdx+frdy) &
                 + (1.-mu_adv)*dtdx*((fraxw+fraxe)+(frays+frayn))


     end do
  end do

debug_3D(:,:,44)=bm(:,:)-bmelt(:,:)

where (i_hpresc(:,:).eq.1)          ! thickness prescribed
   frelax(:,:) = H_presc(:,:)
   arelax(:,:) = 0.
   brelax(:,:) = 0.
   crelax(:,:) = 1.
   drelax(:,:) = 0.
   erelax(:,:) = 0.
end where




stopp = .false.
ntour=0



relax_loop: do while(.not.stopp)
ntour=ntour+1


do j=2,ny-1
   do i=2,nx-1


      reste = (((arelax(i,j)*H(i-1,j) +drelax(i,j)*H(i,j-1)) &
           + (brelax(i,j)*H(i+1,j) + erelax(i,j)*H(i,j+1))) &
           + crelax(i,j)*H(i,j))- frelax(i,j)

if (ntour.eq.1)  debug_3D(i,j,49)=(((arelax(i,j)*H(i-1,j) +drelax(i,j)*H(i,j-1)) &
           + (brelax(i,j)*H(i+1,j) + erelax(i,j)*H(i,j+1))) &
           + crelax(i,j)*H(i,j))


      deltaH(i,j) = reste/crelax(i,j)

   end do
end do

debug_3D(:,:,50)=arelax(:,:)
debug_3D(:,:,51)=brelax(:,:)
debug_3D(:,:,52)=crelax(:,:)
debug_3D(:,:,53)=drelax(:,:)
debug_3D(:,:,54)=erelax(:,:)
debug_3D(:,:,55)=frelax(:,:)

debug_3D(:,:,49)=-deltaH(:,:)


H(:,:)=H(:,:)-deltaH(:,:)

!!$if (ntour.eq.1) then
!!$write(166,*) 'deltaH'
!!$
!!$do j=43,45
!!$write(166,*) 'j=',j
!!$   write(166,*) deltaH(118:122,j)
!!$end do
!!$
!!$end if

! critere d'arret:
! ----------------         

delh=0


do j=2,ny-1
   do i=2,nx-1
      delh=delh+deltaH(i,j)**2
   end do
end do

!!$if (time.gt.22102.) then
!!$   write(166,*) delh,maxval(deltaH(:,:)),maxloc(deltaH(:,:))
!!$end if

if (delh.gt.0.) then
   testh=sqrt(delh)/((nx-2)*(ny-2))
else
   testh=0.
endif
stopp = (testh.lt.1.e-4).or.(ntour.gt.100)

!write(166,*) 'sortie relax',time, ntour,testh,maxval(deltaH(:,:))
end do relax_loop

!write(166,*) time,testh,ntour

!!$! conditions aux limites
!!$

! tests d'asym??trie
!!$it1=2
!!$it2=80
!!$jt1=41
!!$jt2=41
!!$if (H(it1,jt1).ne.H(it2,jt2)) then
!!$   write(6,*)'asymetrie H time',time,'dans resol'
!!$   write(6,*) 'bm',bm(it1,jt1),bm(it2,jt2)
!!$   write(6,*) 'vieuxH',vieuxH(it1,jt1),vieuxH(it2,jt2)
!!$   write(6,*) 'H',H(it1,jt1),H(it2,jt2)
!!$   write(6,*) 'dfx',dfx(it1,jt1),dfx(it1+1,jt1),dfx(it2,jt2),dfx(it2+1,jt2)
!!$   write(6,*) 'diffmx',diffmx(it1,jt1),diffmx(it1+1,jt1),diffmx(it2,jt2),diffmx(it2+1,jt2)
!!$   write(6,*) 'uxbar',uxbar(it1,jt1),uxbar(it1+1,jt1),uxbar(it2,jt2),uxbar(it2+1,jt2)
!!$   write(6,*) 'dfy',dfy(it1,jt1),dfy(it2,jt2),dfy(it1,jt1+1),dfy(it2,jt2+1)
!!$   write(6,*) 'sdx',sdx(it1,jt1),sdx(it2,jt2),sdx(it1+1,jt1),sdx(it2-1,jt2)
!!$   write(6,*) 'sdy',sdy(it1,jt1),sdy(it2,jt2),sdy(it1,jt1+1),sdy(it2,jt2+1)
!!$   write(6,*) 'sdxmy',sdxmy(it1,jt1),sdxmy(it2,jt2),sdxmy(it1+1,jt1),sdxmy(it2+1,jt2)
!!$   write(6,*) 'sdymx',sdymx(it1,jt1),sdymx(it2,jt2),sdymx(it1,jt1+1),sdymx(it2,jt2+1)
!!$   write(6,*)'ddbx',ddbx(it1,jt1),ddby(it2,jt2)
!!$   write(6,*)'ddx(i,j,1)',ddx(it1,it2,1),ddy(it2,it1,1)
!!$   write(6,*) 'arelax',arelax(it1,jt1),arelax(it2,jt2),arelax(it1+1,jt1),arelax(it2-1,jt2)
!!$   write(6,*) 'brelax',brelax(it1,jt1),brelax(it2,jt2),brelax(it1+1,jt1),brelax(it2-1,jt2)
!!$   write(6,*) 'drelax',drelax(it1,jt1),drelax(it2,jt2)
!!$   write(6,*) 'erelax',erelax(it1,jt1),erelax(it2,jt2)
!!$   write(6,*) 'crelax',crelax(it1,jt1),crelax(it2,jt2)
!!$   write(6,*) 'frelax',frelax(it1,jt1),frelax(it2,jt2)
!!$   stop
!!$endif



!   write(166,*) '---------------------------------------------------------------'



return

end subroutine resol_adv_diff_2D_Hpresc

end module reso_adv_diff_2D




