!> \file prop-thermiques-icetemp_mod.f90
!!
!! Implementation of functions and subroutines for temperature calculation 
!!
!<


!///////////////////////////////////////////////////////////////////////
!///////////////////////////////////////////////////////////////////////

!> SUBROUTINE: Thermal_prop_icetemp
!! 
!! Calculate the thermal prop of ice
!!
!! Used modules:
!! - Icetemp_declar
!>

Subroutine Thermal_prop_icetemp(T_m,Tpmp_m,H_m)
  Use Icetemp_declar 

  Implicit None
  !< Arguments
  Real,Dimension(:,:,:),         Intent(In)::T_m         !< Temperature In Ice And Mantle 'O'
  Real,Dimension(:,:,:),         Intent(Inout)::Tpmp_m   !< Pressure Melting Point Temperature In Ice Sheet 'O'
  Real,Dimension(:,:),           Intent(In)::H_m         !< Ice Thickness  'O'

  Nx_m=size(Tpmp_m,1)
  Ny_m=size(Tpmp_m,2)
  Nz_m=size(Tpmp_m,3)

  If (.Not. Allocated(Ct)) then 
     Allocate(Cp(Nx_m,Ny_m,Nz_m),&
          Ct(Nx_m,Ny_m,Nz_m))
  end if

  Do K=1,Nz_m  
     Do J=1,Ny_m
        Do I=1,Nx_m
           !  Temperature De Fusion:Tpmp (En Deg C)
           !  Capacite Calorifique : Cp En J/Kg/K Puis Multiplie Par Ro En J/M3/K
           !  Conductivite : Ct  En J/M/K/An
           !  Attention Pour La Conductivite C'Est La Formule De Yin-Chao Yen

           Ct(I,J,K)=6.62E7
           Cp(I,J,K)=2009. 
           Tpmp_m(I,J,K)=-0.00087*(K-1)*1./(Nz_m-1)*H_m(I,J)!De=1/(Nz_m-1)
           Cp(I,J,K)=(2115.3+7.79293*T_m(I,J,K)) 
           Cp(I,J,K)=Cp(I,J,K)*Ro              ! En J/M3/K
           Ct(I,J,K)=3.1014e8*Exp(-0.0057*(T_m(I,J,K)+273.15)) !En J/M/K/An

        End Do
     End Do
  End Do

End Subroutine Thermal_prop_icetemp


!///////////////////////////////////////////////////////////////////////
!///////////////////////////////////////////////////////////////////////

!> SUBROUTINE: Temp_mer
!! 
!! Calculate temperature of the sea
!! Used modules:
!! - Icetemp_declar
!>
Subroutine Temp_mer(H_m,Flot_m)
  Use Icetemp_declar
  Implicit None 
  !< Arguments
  Real,Dimension(:,:),Intent(In)::H_m  !< Ice Thickness  'O'
  Logical,Dimension(:,:),Intent(In)::Flot_m  !< Vrai Si Flottant (Test D'Archimede) 'O'

  !Instructions
  Do J=1,Ny_m
     Do I=1,Nx_m
        If (Flot_m(I,J)) &
             Tbmer(I,J) = Acof1*S0mer + Bcof1 + Ccof1*H_m(I,J)*Ro/Row

     End Do
  End Do
End Subroutine Temp_mer
!///////////////////////////////////////////////////////////////////////
!///////////////////////////////////////////////////////////////////////








