!> \file climat-forcage_mod-0.4.f90
!!Module pour le calcule du forcage climatique
!<

!> \namespace climat_forcage_mod
!! Module pour le calcule du forcage climatique
!! \author ...
!! \date ...
!! @note Used module
!! @note   - use module3D_phy
!<
module  climat_forcage_mod

! declaration variables 

use module3d_phy

implicit none
character(len=80) :: filin
integer :: nft             !<   nft est le nombre de lignes a lire dans le fichier 
                           !< contenant le forcage climatique
real,dimension(:),allocatable :: tdate          !< time for climate forcing
real,dimension(:),allocatable :: alphat
real,dimension(:),allocatable :: alphap
real,dimension(:),allocatable :: spert


real :: mincoefbmelt            !< butoirs mini 
real :: maxcoefbmelt            !< butoirs maxi de coefbmshelf
character(len=25) :: forcage_file1 !< fichier de forcage 1 (climat)
character(len=25) :: forcage_file2 !< fichier de forcage 2


integer,parameter :: ntr=2 !< nb of snapshot files now explicitely specified 
character(len=80) :: filtr(ntr)

real ttr(ntr)!< la date des tranches (snapshots)  (len=ntr) 
real delta(nx,ny,ntr),deltj(nx,ny,ntr),rapact(nx,ny,ntr)
real  deltatime(nx,ny),deltjtime(nx,ny),rapactime(nx,ny)

contains

!------------------------------------------------------------------------------------------

!> SUBROUTINE: input_clim
!! Routine qui permet d'initialiser les variables climatiques
!>

  subroutine input_clim          !routine qui permet d'initialiser les variables climatiques

    implicit none
    character(len=8) :: control     !label to check clim. forc. file (filin) is usable
    integer l                       !in snapshot files:the first column is the mask, read but not used

    deltatime(:,:)=0.
    deltjtime(:,:)=0.
    rapactime(:,:)=1.

    if (geoplace.eq.'heminor') then

       !     atmospheric temperature gradient
       tempgrad=0.008
       tempgrjul=0.0065


       !pour le nord cette partie est copiee de inputfile-hemicycle.f (3juin04)
       ! fichiers snapshots pour forcage
       !----------------------------------
       filtr(1)=trim(dirnameinp)//'Snapshots-GCM/forclgm-lmd5prese.g50'
       filtr(2)=trim(dirnameinp)//'Snapshots-GCM/forclgm-lmd5futur.g50'

       ! fichiers donnant l'evolution temporelle

       ! this file contains tdate(i),alphat(i),alphap(i),spert(i),i=1,nft      
       ! -------------------hemicycle ------------
       ! lecture du fichier scalaire (sea level)
       !        filin=trim(dirnameinp)//'heminor-forclgm.dat'
       filin=trim(dirnameinp)//'signal-cycle-hemin.dat'

    elseif (geoplace.eq.'hemin40') then

       !     atmospheric temperature gradient
       tempgrad=0.0065   ! 0.0085
       tempgrjul=0.0065

       ! fichiers snapshots pour forcage
       !----------------------------------
       !          filtr(1)=trim(dirnameinp)//'Snapshots-GCM/forcage_hemin40-21k.g40' ! glaciaire std 
       !           filtr(1)=trim(dirnameinp)//'Snapshots-GCM/file_regions_.g40' ! glaciaire regions vincent
       !           filtr(2)=trim(dirnameinp)//'Snapshots-GCM/file_regions_2.g40' ! pour glaciaire permanent
       !          filtr(2)=trim(dirnameinp)//'Snapshots-GCM/forclgm-reg_oc2-00k.g40'
       !          filtr(2)=trim(dirnameinp)//'Snapshots-GCM/no_forcage_hn40-0k.g40' ! climat act a 0ky
       !           filtr(2)=trim(dirnameinp)//'Snapshots-GCM/file_regions_2act.g40' ! pour climat act permanent

       !cdc forcage modele IPSL LSCE Cycle
       !            filtr(1)=TRIM(DIRNAMEINP)//'Snapshots-GCM/forcage_IPSL-21k.g40'
       !            filtr(2)=TRIM(DIRNAMEINP)//'Snapshots-GCM/no_forcage_hn40-0k.g40' !climat act a 0ka
       filtr(1)=TRIM(DIRNAMEINP)//'Snapshots-GCM/'//TRIM(forcage_file1)
       filtr(2)=TRIM(DIRNAMEINP)//'Snapshots-GCM/'//TRIM(forcage_file2)
       print*,'fichier de forcage 1',TRIM(DIRNAMEINP)//'Snapshots-GCM/'//TRIM(forcage_file1)
       print*,'fichier de forcage 2',TRIM(DIRNAMEINP)//'Snapshots-GCM/'//TRIM(forcage_file2)

       ! fichiers donnant l'evolution temporelle
       ! -----------------------------------------
       filin=trim(dirforcage)//'signal-cycle-hemin.dat'

    else if ((geoplace.eq.'anteis1').or.(geoplace.eq.'ant20km')) then


       !     atmospheric temperature gradient
       tempgrad=0.0065   ! 0.0085
       tempgrjul=0.0065

       !cdc forcage modele IPSL LSCE Cycle
       !            filtr(1)=TRIM(DIRNAMEINP)//'Snapshots-GCM/forcage_IPSL-21k.g40'
       !            filtr(2)=TRIM(DIRNAMEINP)//'Snapshots-GCM/no_forcage_hn40-0k.g40' !climat act a 0ka
       filtr(1)=TRIM(DIRNAMEINP)//'Snapshots-GCM/'//TRIM(forcage_file1)
       filtr(2)=TRIM(DIRNAMEINP)//'Snapshots-GCM/'//TRIM(forcage_file2)
       print*,'fichier de forcage 1',TRIM(DIRNAMEINP)//'Snapshots-GCM/'//TRIM(forcage_file1)
       print*,'fichier de forcage 2',TRIM(DIRNAMEINP)//'Snapshots-GCM/'//TRIM(forcage_file2)

       ! fichiers donnant l'evolution temporelle
       ! ---------------------------- ------------
       filin=trim(dirforcage)//'signal-cycle-hemin.dat' !forcage vostok en rapport : a creer

    endif !fin du test sur geoplace

    ! lecture des fichiers snapsots pour n'importe quel  geoplace
    ! -------------------------------------------------------------
    write(6,*) 'fichiers snapshots'
    do k=1,ntr
       write(6,*) filtr(k)
       open(num_forc,file=filtr(k))
       read(num_forc,*) ttr(k)
       read(num_forc,*)
       read(num_forc,*)
       do j=1,ny
          do i=1,nx
             read(num_forc,*) l,rapact(i,j,k),delta(i,j,k),deltj(i,j,k)
          end do
       end do
       close(num_forc)
    end do
    ttr(1)=-500000. !ttr est la date des tranches 3d-1.h
    ttr(2)=0

    ! lecture des fichiers d'evolution pour tout geoplace
    ! ----------------------------------------------------
    open(num_forc,file=filin,status='old')
    !        print*,nft
    read(num_forc,*) control,nft
    print*,'control',control,nft
    ! determination of file size (line nb), allocation of perturbation array

    if (control.ne.'nb_lines') then
       write(6,*) filin,'indiquer le nb de ligne en debut de fichier:'
       write(6,*) 'le nb de lignes et le label de control nb_lines'
       stop
    endif

    if (.not.allocated(tdate)) THEN
       allocate(tdate(nft),stat=err)
       if (err/=0) then
          print *,"erreur  l'allocation du tableau tdate",err
          stop 4
       end if
    end if

    if (.not.allocated(alphat)) THEN
       allocate(alphat(nft),stat=err)
       if (err/=0) then
          print *,"erreur  l'allocation du tableau tpert",err
          stop 4
       end if
    end if

    if (.not.allocated(alphap)) THEN
       allocate(alphap(nft),stat=err)
       if (err/=0) then
          print *,"erreur  l'allocation du tableau tpert",err
          stop 4
       end if
    end if

    if (.not.allocated(spert)) THEN
       allocate(spert(nft),stat=err)
       if (err/=0) then
          print *,"erreur  l'allocation du tableau spert",err
          stop 4
       end if
    end if

    do i=1,nft
       read(num_forc,*) tdate(i),alphat(i),alphap(i),spert(i)
       !          print*,tdate(i),alphat(i),alphap(i),spert(i)
    end do

    close(num_forc)


  end subroutine input_clim


!--------------------------------------------------------------------------------
!> SUBROUTINE: init_forclim
!! Initialisation des parametres du forcage climatique au cours du temps
!>

subroutine init_forclim


namelist/clim_forcage/forcage_file1,forcage_file2,mincoefbmelt,maxcoefbmelt

rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
read(num_param,clim_forcage)


! formats pour les ecritures dans 42
428 format(A)

write(num_rep_42,428)'!___________________________________________________________' 
write(num_rep_42,428) '&clim_forcage                                ! nom du bloc '
write(num_rep_42,*)
write(num_rep_42,*) 'forcage_file1 = ', forcage_file1
write(num_rep_42,*) 'forcage_file2 = ', forcage_file2
write(num_rep_42,*) 'mincoefbmelt = ', mincoefbmelt
write(num_rep_42,*) 'maxcoefbmelt = ', maxcoefbmelt
write(num_rep_42,*)'/'                            
write(num_rep_42,*)
return
end subroutine init_forclim


!---------------------------------------------------------------------
!> SUBROUTINE: forclim
!! Routine pour le calcul du forcage climatique au cours du temps
!>

subroutine forclim

implicit none
real :: coeft,coefp,coeftime       !coeftime is not used
integer :: l                       !dumm index for loops on snapsots files  l=itr,ntr-1
integer itr                        !nb of the current snapshot file (change with time)


if(time.le.tdate(1)) then   ! time avant le debut du fichier forcage
   sealevel=spert(1)
   coeft=alphat(1)
   coefp=alphap(1)
   ift=1

else if (time.ge.tdate(nft)) then     ! time apres la fin du fichier forcage
   sealevel=spert(nft)
   coeft=alphat(nft)
   coefp=alphap(nft)
   ift=nft

else                         ! time en general
   
   ift = 1      
   do i=ift,nft-1  ! interpolation entre i et i+1

      if((time.ge.tdate(i)).and.(time.lt.tdate(i+1))) then  ! cas general
          sealevel=spert(i)+(spert(i+1)-spert(i))*    &
                 (time-tdate(i))/(tdate(i+1)-tdate(i))

          coeft=alphat(i)+(alphat(i+1)-alphat(i))*      &
                 (time-tdate(i))/(tdate(i+1)-tdate(i))

          coefp=alphap(i)+(alphap(i+1)-alphap(i))*      &
                 (time-tdate(i))/(tdate(i+1)-tdate(i))
     
!        print*,'sea,coeft,coefp', sealevel,coeft,coefp
          ift=i
          goto 100
         endif

      end do
endif
100   continue

!          print*,'coefft p',coeft,coefp
!=forcage du climat

if(time.le.ttr(1)) then   ! time en dehors des limites du fichier forcage
   do j=1,ny
      do i=1,nx
         deltatime(i,j)=delta(i,j,1)
         deltjtime(i,j)=deltj(i,j,1)
         rapactime(i,j)=rapact(i,j,1)
      end do
   end do
   itr=1

else if (time.ge.ttr(ntr)) then        ! mis a 0
   do j=1,ny
      do i=1,nx
         deltatime(i,j)=delta(i,j,ntr)
         deltjtime(i,j)=deltj(i,j,ntr)
         rapactime(i,j)=rapact(i,j,ntr)
      end do
   end do
   itr=ntr

else               !  interpolation entre l et l+1 : cas general
   itr=max(itr,1)

   do l=itr,ntr-1    ! interpolation entre i et i+1
      
      if((time.ge.ttr(l)).and.(time.lt.ttr(l+1))) then  ! test tranche
         
         coeftime= (time-ttr(l))/(ttr(l+1)-ttr(l))
         do j=1,ny
            do i=1,nx
               deltatime(i,j)=delta(i,j,l)+             &
                    (delta(i,j,l+1)-delta(i,j,l))*coeft

               deltjtime(i,j)=deltj(i,j,l)+             &
                    (deltj(i,j,l+1)-deltj(i,j,l))*coeft

               rapactime(i,j)=rapact(i,j,l)+            &
                    (rapact(i,j,l+1)-rapact(i,j,l))*coefp
            end do
         end do
         itr=l
         goto 200

      endif        ! fin du test sur la tranche
   end do
endif   ! fin du test avant,apres,milieu

200   continue

deltatime(:,:)=0.
deltjtime(:,:)=0.
rapactime(:,:)=1.

coefbmshelf=1. ! modif pour test de sensibilite (tof 20 avril 01)
coefbmshelf=max(coefbmshelf,mincoefbmelt)
coefbmshelf=min(coefbmshelf,maxcoefbmelt)

if (geoplace(1:5).eq.'hemin') then
   call accum7()

else if ((geoplace.eq.'anteis1').or.(geoplace.eq.'ant20km')) then
   call massb_anteis_forcage()

else
   print*,"partie en travaux climat forcage autre que sur heminord"
   print*,"geoplace ",geoplace
endif


seasea=sealevel        ! pour des sorties

end subroutine forclim

end module  climat_forcage_mod
