module climat_coupl_atm_mod                   

! forcage avec champs mensuels
! equilibre climatique avec 1 seul snapshot climatique
! lecture fichiers snapshots climat Temp et Precip mensuels (2 fichiers)
! nouvelle version avec liste des variables utilises par le module
! C. Dumas 02/2015

USE module3D_phy,only:nx,ny,sealevel,seasea,Tann,Tjuly,Tmois,Pmois,acc,coefbmshelf,ro,num_param,num_rep_42,dirforcage,num_forc,time
!use netcdf
!use io_netcdf_grisli
!USE printtable
       
implicit none
real :: mincoefbmelt            ! butoirs mini 
real :: maxcoefbmelt            ! butoirs maxi de coefbmshelf
character(len=80) :: filforc    ! nom du fichier forcage
real,dimension(:),allocatable :: tdate          ! time for climate forcing
real,dimension(:),allocatable :: spert          ! sea level perturbation
integer :: nft             !   nft nbr lignes a lire dans le fichier contenant le forcage climatique

integer,parameter :: nbmois=12
logical :: snow ! Flag pour selectionner precip liq ou acc neige


real,dimension(nx,ny,nbmois) :: Tm                    ! temperature mensuelle
real,dimension(nx,ny,nbmois) :: Pm                    ! precipitation mensuelle

real,dimension(nx,ny,nbmois) :: Tm_surf                 ! surface temperature (after topo. correction) 13 nbmois pour modulo
real,dimension(nx,ny,nbmois) :: Pm_surf                 ! surface precipitation (after topo. correction)

real,dimension(nx,ny) :: ZS                             !< surface topography above sea level

real,dimension(nx,ny,nbmois) :: lapserate                 ! lapse rate
real :: psolid=2.                                      ! temp limit between liquid and solid precip

character(len=200)  :: filin                            ! nom temporaire
!character(len=100) :: file_temporel                    ! forcage temporel
!character(len=100),dimension(ntr) :: filtr_t, filtr_p   ! snapshot file name
character(len=100) :: filtr_t1,filtr_p1                 ! snapshot file name


contains

! 2=lecture des inputs
!--------------------

subroutine input_clim 					! routine qui permet d'initialiser les variables climatiques

! variables locales
!-------------------

  implicit none
  integer :: i,j,mo,ti,tj  
  character(len=8) :: control	! label to check clim. forc. file (filin) is usable
  integer :: l 			! In snapshot files:the first column is the mask, read but not used
  real :: T_surf_ref            ! variable de travail calcul temp a l'instant t a la surface S
  integer :: err                ! pour allocation tableaux
  real :: toto
 

! fichiers donnant l'evolution temporelle
! -----------------------------------------
! Pour rappel hemin40 :
! fichier forcage : signal-cycle-hemin.dat
! Pour anteis :
! fichier forcage : forcmodif4cycles.dat
  filin=trim(dirforcage)//trim(filforc) 

! lecture des fichiers d'evolution pour tout geoplace
! ----------------------------------------------------
  open(num_forc,file=filin,status='old')
!        print*,nft
  read(num_forc,*) control,nft
  print*,'control',control,nft
! determination of file size (line nb), allocation of perturbation array

if (control.ne.'nb_lines') then
         write(6,*) filin,'indiquer le nb de ligne en debut de fichier:'
         write(6,*) 'le nb de lignes et le label de control nb_lines'
         stop
endif

      if (.not.allocated(tdate)) THEN
       allocate(tdate(nft),stat=err)
         if (err/=0) then
           print *,"erreur  l'allocation du tableau tdate",err
           stop 4
         end if
    end if

      if (.not.allocated(spert)) THEN
       allocate(spert(nft),stat=err)
         if (err/=0) then
           print *,"erreur  l'allocation du tableau spert",err
           stop 4
         end if
    end if
        
         do i=1,nft
           read(num_forc,*) tdate(i),toto,toto,spert(i)
!          print*,tdate(i),alphat(i),alphap(i),spert(i)
         end do

      close(num_forc)
  
end subroutine input_clim



!-------------------------------------------------------------------------------
! lecture de la namelist
subroutine init_forclim

  real,dimension(12) :: lect_lapse       ! lecture lapse rate
  integer :: mo


  namelist/clim_coupl_atm/mincoefbmelt,maxcoefbmelt,filforc,snow
  namelist/lapse_month/lect_lapse

rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
read(num_param,clim_coupl_atm)

! formats pour les ecritures dans 42
428 format(A)

write(num_rep_42,428)'!___________________________________________________________' 
write(num_rep_42,428) '&clim_coupl_atm                              ! nom du bloc '
write(num_rep_42,*)
write(num_rep_42,*) 'mincoefbmelt = ', mincoefbmelt
write(num_rep_42,*) 'maxcoefbmelt = ', maxcoefbmelt
write(num_rep_42,'(A,A)') ' filforc      = ', filforc
write(num_rep_42,*) 'snow = ', snow
write(num_rep_42,*)'/'
write(num_rep_42,*)'! mincoefbmelt : borne mini de coefbmshelf'
write(num_rep_42,*)'! maxcoefbmelt : borne maxi de coefbmshelf'
write(num_rep_42,*)'! filforc : fichier de forcage nivx des mers'
write(num_rep_42,*)'! snow   : .true. si neige, .false. si precip'       
write(num_rep_42,*)


! lapserate
!  namelist/lapse_month/lect_lapse
  rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
  read(num_param,lapse_month)
  write(num_rep_42,428)'!___________________________________________________________' 
  write(num_rep_42,428) '&lapse_month                                   !  module climat_forcage_mois_mod'
  write(num_rep_42,'(A,12(f0.2,","))')  'lapse_month  = ', lect_lapse
  write(num_rep_42,*)'/'                      
  write(num_rep_42,428) '! laspe rates janvier -> decembre en deg/km'
  write(num_rep_42,*)

! pour repasser en deg/m et copier dans lapserate
  do mo=1,nbmois
     lapserate(:,:,mo)=lect_lapse(mo)/1000.
  enddo

! coefbmshelf coefficient pour la fusion basale sous les ice shelves
! ici pas d'evolution temporelle donc coefbmshelf est fixe
  coefbmshelf=1.

end subroutine init_forclim


!---------------------------------------------------------------------
!forcage climatique au cours du temps
subroutine forclim

implicit none

  integer i,ift
  
  
 if(time.le.tdate(1)) then   ! time avant le debut du fichier forcage
   sealevel=spert(1)
   ift=1
else if (time.ge.tdate(nft)) then     ! time apres la fin du fichier forcage
   sealevel=spert(nft)
   ift=nft
else                         ! time en general
   
   ift = 1      
   do i=ift,nft-1  ! interpolation entre i et i+1

      if((time.ge.tdate(i)).and.(time.lt.tdate(i+1))) then  ! cas general
          sealevel=spert(i)+(spert(i+1)-spert(i))*    &
                 (time-tdate(i))/(tdate(i+1)-tdate(i))
          ift=i
          goto 100
       endif

    end do
endif
100   continue 


coefbmshelf=1. ! modif pour test de sensibilite (tof 20 avril 01)
coefbmshelf=max(coefbmshelf,mincoefbmelt)
coefbmshelf=min(coefbmshelf,maxcoefbmelt)

!**********************************************************************
! transmission des champs climatiques de iLOVECLIM via pointeurs Tann, Acc, Tjuly, Tmois, Pmois, Bm
! Temperatures Annuels, jja et accumulation de neige
!**********************************************************************
! calcul de l'accumulation si on utilise les precips liquides :
if (.not.snow) then
!    if (annual.eq.1) then ! la version avec Tann et Tjuly est obsolete !
!       call accum_annual ! calcul acc a partir de Tann et Tjuly
!    else if (annual.eq.2) then
      acc(:,:)=sum(Pmois,dim=3,mask=Tmois < psolid) ! /12.
      acc(:,:)=acc(:,:)*1000./ro ! m de glace/an
!    endif
endif
    Tann(:,:)=SUM(Tmois(:,:,:),dim=3)/12.
    Tjuly(:,:)=Tmois(:,:,7)

seasea=sealevel        ! pour les sorties

END subroutine forclim


end module climat_coupl_atm_mod
